<?php
/**
 * Plugin Name: WordPress Access Control
 * Plugin URI: http://brandonwamboldt.ca/plugins/members-only-menu-plugin/
 * Author: Brandon Wamboldt
 * Author URI: http://brandonwamboldt.ca/
 * Version: 2.0
 * Description: Allows you to limit access to pages to members, non-members or specific roles, having unauthorized users redirected to the login page or a specified URL. Pages can still be added to WordPress navigation menus and will only show up for users with the required access. 
 */

add_action( 'wp', array( 'WordPressAccessControl', 'check_for_members_only' ) );
add_action( 'wp', array( 'WordPressAccessControl', 'check_for_nonmembers_only' ) );
add_action( 'add_meta_boxes', array( 'WordPressAccessControl', 'add_wp_access_meta_boxes' ) );
add_action( 'save_post', array( 'WordPressAccessControl', 'save_postdata' ) );

add_filter( 'get_pages', array( 'WordPressAccessControl', 'get_pages' ) );

// We check to see if the class exists because it isn't part of pre WordPress 3 systems
if ( class_exists( 'Walker_Nav_Menu' ) ) {
	add_filter( 'wp_nav_menu_args', array( 'WordPressAccessControl', 'wp_nav_menu_args' ) );

	/**
	 * A custom walker that checks our conditions to make sure the user has
	 * permission to view the page linked to from the menu item
	 *
	 * @author Brandon Wamboldt
	 * @package WordPress
	 * @subpackage WordPressAccessControl
	 */
	class WPAC_Nav_Menu_Walker extends Walker_Nav_Menu
	{
		var $in_private = false;
		var $private_lvl = 0;

		function start_lvl( & $output, $depth )
		{
			if ( ! $this->in_private ) {
				$indent = str_repeat( '    ', $depth );
				$output .= "\n$indent<ul class=\"sub-menu\"><li style='display:none;'></li>\n";
			}
		}

		function end_lvl( & $output, $depth )
		{
			if ( ! $this->in_private ) {
				$indent = str_repeat( '    ', $depth );
				$output .= "$indent</ul>\n";
			}
		}  

		function start_el( & $output, $item, $depth, $args )
		{
			global $wp_query;

			if ( WordPressAccessControl::check_conditions( $item->object_id ) ) {
				if ( ! $this->in_private ) {
					$indent = ( $depth ) ? str_repeat( "\t", $depth ) : '';

					$class_names = $value = '';

					$classes = empty( $item->classes ) ? array() : (array) $item->classes;

					$class_names = join( ' ', apply_filters( 'nav_menu_css_class', array_filter( $classes ), $item ) );
					$class_names = ' class="' . esc_attr( $class_names ) . '"';

					$output .= $indent . '<li id="menu-item-'. $item->ID . '"' . $value . $class_names .'>';

					$attributes  = ! empty( $item->attr_title ) ? ' title="'  . esc_attr( $item->attr_title ) .'"' : '';
					$attributes .= ! empty( $item->target )     ? ' target="' . esc_attr( $item->target     ) .'"' : '';
					$attributes .= ! empty( $item->xfn )        ? ' rel="'    . esc_attr( $item->xfn        ) .'"' : '';
					$attributes .= ! empty( $item->url )        ? ' href="'   . esc_attr( $item->url        ) .'"' : '';

					$item_output = $args->before;
					$item_output .= '<a'. $attributes .'>';
					$item_output .= $args->link_before . apply_filters( 'the_title', $item->title, $item->ID ) . $args->link_after;
					$item_output .= '</a>';
					$item_output .= $args->after;

					$output .= apply_filters( 'walker_nav_menu_start_el', $item_output, $item, $depth, $args );
				}
			} else {
				$this->in_private = true;
				$this->private_lvl++;
			}
		}

		function end_el( & $output, $item, $depth )
		{
			if ( WordPressAccessControl::check_conditions( $item->object_id ) ) {
				if ( ! $this->in_private ) {
					$output .= "</li>\n";
				}
			} else {
				$this->private_lvl--;

				if ( $this->private_lvl == 0 ) {
					$this->in_private = false;
				}
			}
		}
	}
}

// We check to see if the class exists because it may get removed in the future
if ( class_exists( 'Walker_Page' ) ) {
	add_filter( 'wp_page_menu_args', array( 'WordPressAccessControl' , 'wp_page_menu_args' ) );

	/**
	 * A custom walker that checks our conditions to make sure the user has
	 * permission to view the page linked to from the menu item
	 *
	 * @author Brandon Wamboldt
 	 * @package WordPress
	 * @subpackage WordPressAccessControl
	 */
	class WPAC_Page_Walker extends Walker_Page
	{
		var $in_private = false;
		var $private_lvl = 0;

		function start_lvl( & $output, $depth )
		{
			if ( ! $this->in_private ) {
				$indent = str_repeat( '    ', $depth );
				$output .= "\n$indent<ul class=\"sub-menu\"><li style='display:none;'></li>\n";
			}
		}

		function end_lvl( & $output, $depth )
		{
			if ( ! $this->in_private ) {
				$indent = str_repeat( '    ', $depth );
				$output .= "$indent</ul>\n";
			}
		}

		function start_el( & $output, $page, $depth, $args, $current_page )
		{
			global $wp_query;

			if ( WordPressAccessControl::check_conditions( $page->ID ) ) {
				if ( ! $this->in_private ) {
					$indent = ( $depth ) ? str_repeat( "\t", $depth ) : '';

					extract( $args, EXTR_SKIP );

					$css_class = array( 'page_item', 'page-item-' . $page->ID );

					if ( $current_page != '' ) {
						$_current_page = get_page( $current_page );

						if ( isset( $_current_page->ancestors ) && in_array( $page->ID, (array) $_current_page->ancestors ) ) {
							$css_class[] = 'current_page_ancestor';
						}

						if ( $page->ID == $current_page ) {
                            $css_class[] = 'current_page_item';
						} elseif ( $_current_page && $page->ID == $_current_page->post_parent ) {
							$css_class[] = 'current_page_parent';
						}
					} elseif ( $page->ID == get_option('page_for_posts') ) {
						$css_class[] = 'current_page_parent';
					}

					$css_class = implode( ' ', apply_filters( 'page_css_class', $css_class, $page ) );

					$output .= $indent . '<li class="' . $css_class . '"><a href="' . get_page_link( $page->ID ) . '" title="' . esc_attr( wp_strip_all_tags( apply_filters( 'the_title', $page->post_title, $page->ID ) ) ) . '">' . $link_before . apply_filters( 'the_title', $page->post_title, $page->ID ) . $link_after . '</a>';

					if ( ! empty( $show_date ) ) {
						if ( 'modified' == $show_date ) {
							$time = $page->post_modified;
						} else {
							$time = $page->post_date;
						}

						$output .= ' ' . mysql2date( $date_format, $time );
					}
				}
			} else {
				$this->in_private = true;
				$this->private_lvl++;
            }
		}

		function end_el( & $output, $page, $depth )
		{
			if ( WordPressAccessControl::check_conditions( $page->ID ) ) {
				if ( ! $this->in_private ) {
					$output .= "</li>\n";
				}
			} else {
				$this->private_lvl--;

				if ( $this->private_lvl == 0 ) {
					$this->in_private = false;
				}
			}
		}
	}
}

/**
 * The main plugin
 * 
 * @author Brandon Wamboldt
 * @package WordPress
 * @subpackage WordPressAccessControl
 */
class WordPressAccessControl 
{
	function check_conditions( $page_id )
	{
		global $wp_roles, $current_user;

		if ( is_user_logged_in() ) {
			get_currentuserinfo();

			$allowed_roles = maybe_unserialize( get_post_meta( $page_id, '_wpac_restricted_to', true ) );

			if ( empty( $allowed_roles ) ) {
				$allowed_roles_tmp = $wp_roles->get_names();
				$allowed_roles = array();

				foreach ( $allowed_roles_tmp as $role => $garbage ) {
					$allowed_roles[] = $role;
				}
			}

			$intersections = array_intersect( $current_user->roles, $allowed_roles );

			if ( empty( $intersections ) ) {
				$role = false;
			} else {
				$role = true;
			}
		} else {
			$role = true;
		}

		$is_members_only = get_post_meta( $page_id, '_wpac_is_members_only', true );
		$is_nonmembers_only = get_post_meta( $page_id, '_wpac_is_nonmembers_only', true );

		if ( ( is_user_logged_in() && $is_members_only && $role ) || ( ! is_user_logged_in() && $is_nonmembers_only ) || ( ! $is_members_only && ! $is_nonmembers_only ) ) {
			return true;
		} else {
			return false;
		}
	}

	function check_for_members_only()
	{
		global $post;

		if ( get_post_meta( $post->ID, '_wpac_is_members_only', true ) && !self::check_conditions( $post->ID ) ) {
			$redirect_to = get_post_meta( $post->ID, '_wpac_members_redirect_to', true );

			if ( empty( $redirect_to ) ) {
				header( 'Location: ' . get_bloginfo( 'wpurl' ) . '/wp-login.php' );
			} else {
				header( 'Location: ' . $redirect_to );
			}

			exit();
		}
	}

	function check_for_nonmembers_only()
	{
		global $post;

		if ( get_post_meta( $post->ID, '_wpac_is_nonmembers_only', true ) && ! self::check_conditions( $post->ID ) ) {
			$redirect_to = get_post_meta( $post->ID, '_wpac_nonmembers_redirect_to', true );

			if ( empty( $redirect_to ) ) {
				header( 'Location: ' . get_bloginfo( 'wpurl' ) . '/index.php' );
			} else {
				header( 'Location: ' . $redirect_to );
			}

			exit();
		}
	}

	function add_wp_access_meta_boxes()
	{
		$supported_pages = apply_filters( 'wp_access_supported_pages', array( 'page' ) );

		foreach ( $supported_pages as $page_type ) {
			add_meta_box( 'wpac_controls_meta', 'WordPress Access Control', array( 'WordPressAccessControl', 'add_wpac_meta_box' ), $page_type, 'side', 'high' );
		}
	}

	function add_wpac_meta_box()
	{
		global $post;

		$is_members_only = get_post_meta( $post->ID, '_wpac_is_members_only', true );
		$is_nonmembers_only = get_post_meta( $post->ID, '_wpac_is_nonmembers_only', true );

		if ( $is_members_only == 'true' ) {
			$is_members_only = 'checked="checked"';
		}

		if ( $is_nonmembers_only == 'true' ) {
			$is_nonmembers_only = 'checked="checked"';
		}

		include( dirname( __FILE__ ) . '/templates/meta_box.php' );
	}

	function save_postdata( $post_id )
	{
		if ( ! isset($_POST['members_only_nonce'] ) ) {
			return $post_id;
		}

		if ( ! wp_verify_nonce( $_POST['members_only_nonce'], 'members_only' ) ) {
			return $post_id;
		}

		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
			return $post_id;
		}

		if ( isset( $_POST['members_only'] ) && $_POST['members_only'] == 'true' ) {
			update_post_meta( $post_id, '_wpac_is_members_only', 'true' );

			if ( isset( $_POST['wpac_restricted_to'] ) && ! empty( $_POST['wpac_restricted_to'] ) ) {
				update_post_meta( $post_id, '_wpac_restricted_to', serialize( $_POST['wpac_restricted_to'] ) );
			} else {
				delete_post_meta( $post_id, '_wpac_restricted_to' );
			}
		} else {
			delete_post_meta( $post_id, '_wpac_is_members_only' );
		}

		if ( isset( $_POST['nonmembers_only'] ) && $_POST['nonmembers_only'] == 'true' ) {
			update_post_meta( $post_id, '_wpac_is_nonmembers_only', 'true' );
		} else {
			delete_post_meta( $post_id, '_wpac_is_nonmembers_only' );
		}

		if ( isset( $_POST['wpac_members_redirect_to'] ) ) {
			update_post_meta( $post_id, '_wpac_members_redirect_to', $_POST['wpac_members_redirect_to'] );
		}

		if ( isset( $_POST['wpac_nonmembers_redirect_to'] ) ) {
			update_post_meta( $post_id, '_wpac_nonmembers_redirect_to', $_POST['wpac_nonmembers_redirect_to'] );
		}

		return $post_id;
	}

	function wp_nav_menu_args( $args )
	{
		$args['walker'] = new WPAC_Nav_Menu_Walker();
		return $args;
	}

	function wp_page_menu_args( $args )
	{
		// Only remove the walker if it is ours
		if ( isset( $args['walker'] ) && get_class( $args['walker']) == 'WPAC_Nav_Menu_Walker' ) {
			$args['walker'] = new WPAC_Page_Walker();
		}

		return $args;
	}
    
	/**
	 * This hooks in at a higher level to make sure functions like 
	 * wp_list_pages won't return members only pages.
	 * 
	 * @param $pages
	 * @return array
	 */
	function get_pages( $pages )
	{
		$auth = is_user_logged_in();

		foreach ( $pages as $key => $page ) {
			$is_members_only = get_post_meta( $page->ID, '_wpac_is_members_only', true );
			$is_nonmembers_only = get_post_meta( $page->ID, '_wpac_is_nonmembers_only', true );

			if ( $is_members_only && ! $auth ) {
				unset( $pages[$key] );
			}

			if ( $is_nonmembers_only && $auth ) {
				unset( $pages[$key] );
			}
		}

		return $pages;
	}
}